@@ -194,8 +194,8 @@ gdata_service_init (GDataService *self)
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_SERVICE, GDataServicePrivate);
 	self->priv->session = _gdata_service_build_session ();
 
-	/* Debug log handling */
-	g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, (GLogFunc) debug_handler, self);
+	/* Log handling for all message types except debug */
+	g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR | G_LOG_LEVEL_INFO | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_WARNING, (GLogFunc) debug_handler, self);
 
 	/* Proxy the SoupSession's proxy-uri and timeout properties */
 	g_signal_connect (self->priv->session, "notify::proxy-uri", (GCallback) notify_proxy_uri_cb, self);
@@ -539,9 +539,13 @@ _gdata_service_build_message (GDataService *self, GDataAuthorizationDomain *doma
 {
 	SoupMessage *message;
 	GDataServiceClass *klass;
+	SoupURI *_uri;
 
-	/* Create the message */
-	message = soup_message_new (method, uri);
+	/* Create the message. Allow changing the HTTPS port just for testing. */
+	_uri = soup_uri_new (uri);
+	soup_uri_set_port (_uri, _gdata_service_get_https_port ());
+	message = soup_message_new_from_uri (method, _uri);
+	soup_uri_free (_uri);
 
 	/* Make sure subclasses set their headers */
 	klass = GDATA_SERVICE_GET_CLASS (self);
@@ -639,7 +643,9 @@ _gdata_service_actually_send_message (SoupSession *session, SoupMessage *message
 	g_assert (message->status_code != SOUP_STATUS_NONE);
 
 	if (message->status_code == SOUP_STATUS_CANCELLED ||
-	    (message->status_code == SOUP_STATUS_IO_ERROR && cancellable != NULL && g_cancellable_is_cancelled (cancellable) == TRUE)) {
+	    ((message->status_code == SOUP_STATUS_IO_ERROR || message->status_code == SOUP_STATUS_SSL_FAILED ||
+	      message->status_code == SOUP_STATUS_CANT_CONNECT || message->status_code == SOUP_STATUS_CANT_RESOLVE) &&
+	     cancellable != NULL && g_cancellable_is_cancelled (cancellable) == TRUE)) {
 		/* We hackily create and cancel a new GCancellable so that we can set the error using it and therefore save ourselves a translatable
 		 * string and the associated maintenance. */
 		GCancellable *error_cancellable = g_cancellable_new ();
@@ -688,6 +694,9 @@ _gdata_service_send_message (GDataService *self, SoupMessage *message, GCancella
 			return SOUP_STATUS_NONE;
 		}
 
+		/* Allow overriding the URI for testing. */
+		soup_uri_set_port (new_uri, _gdata_service_get_https_port ());
+
 		soup_message_set_uri (message, new_uri);
 		soup_uri_free (new_uri);
 
@@ -1280,10 +1289,9 @@ gdata_service_insert_entry_finish (GDataService *self, GAsyncResult *async_resul
 		return NULL;
 
 	entry = g_simple_async_result_get_op_res_gpointer (result);
-	if (entry != NULL)
-		return g_object_ref (entry);
+	g_assert (entry != NULL);
 
-	g_assert_not_reached ();
+	return g_object_ref (entry);
 }
 
 /**
@@ -1476,10 +1484,9 @@ gdata_service_update_entry_finish (GDataService *self, GAsyncResult *async_resul
 		return NULL;
 
 	entry = g_simple_async_result_get_op_res_gpointer (result);
-	if (entry != NULL)
-		return g_object_ref (entry);
+	g_assert (entry != NULL);
 
-	g_assert_not_reached ();
+	return g_object_ref (entry);
 }
 
 /**
@@ -1957,6 +1964,40 @@ _gdata_service_fix_uri_scheme (const gchar *uri)
 	return g_strdup (uri);
 }
 
+/**
+ * _gdata_service_get_https_port:
+ *
+ * Gets the destination TCP/IP port number which libgdata should use for all outbound HTTPS traffic.
+ * This defaults to 443, but may be overridden using the <code class="literal">LIBGDATA_HTTPS_PORT</code>
+ * environment variable. This is intended to allow network traffic to be redirected to a local server for
+ * unit testing, with a listening port above 1024 so the tests don't need root privileges.
+ *
+ * The value returned by this function may change at any time (e.g. between unit tests), so callers must not cache the result.
+ *
+ * Return value: port number to use for HTTPS traffic
+ */
+guint
+_gdata_service_get_https_port (void)
+{
+	const gchar *port_string;
+
+	/* Allow changing the HTTPS port just for testing. */
+	port_string = g_getenv ("LIBGDATA_HTTPS_PORT");
+	if (port_string != NULL) {
+		const gchar *end;
+
+		guint64 port = g_ascii_strtoull (port_string, (gchar **) &end, 10);
+
+		if (port != 0 && *end == '\0') {
+			g_debug ("Overriding message port to %" G_GUINT64_FORMAT ".", port);
+			return port;
+		}
+	}
+
+	/* Return the default. */
+	return 443;
+}
+
 /*
  * debug_handler:
  *
@@ -2112,7 +2153,17 @@ _gdata_service_get_log_level (void)
 SoupSession *
 _gdata_service_build_session (void)
 {
-	SoupSession *session = soup_session_sync_new_with_options (SOUP_SESSION_SSL_USE_SYSTEM_CA_FILE, TRUE, NULL);
+	SoupSession *session;
+	gboolean ssl_strict = TRUE;
+
+	/* Iff LIBGDATA_LAX_SSL_CERTIFICATES=1, relax SSL certificate validation to allow using invalid/unsigned certificates for testing. */
+	if (g_strcmp0 (g_getenv ("LIBGDATA_LAX_SSL_CERTIFICATES"), "1") == 0) {
+		ssl_strict = FALSE;
+	}
+
+	session = soup_session_new_with_options ("ssl-strict", ssl_strict,
+	                                         "timeout", 0,
+	                                         NULL);
 
 #ifdef HAVE_GNOME
 	soup_session_add_feature_by_type (session, SOUP_TYPE_GNOME_FEATURES_2_26);
