@@ -409,9 +409,13 @@ static SoupMessage *
 build_message (GDataUploadStream *self, const gchar *method, const gchar *upload_uri)
 {
 	SoupMessage *new_message;
+	SoupURI *_uri;
 
 	/* Build the message */
-	new_message = soup_message_new (method, upload_uri);
+	_uri = soup_uri_new (upload_uri);
+	soup_uri_set_port (_uri, _gdata_service_get_https_port ());
+	new_message = soup_message_new_from_uri (method, _uri);
+	soup_uri_free (_uri);
 
 	/* We don't want to accumulate chunks */
 	soup_message_body_set_accumulate (new_message->request_body, FALSE);
@@ -1055,8 +1059,6 @@ write_next_chunk (GDataUploadStream *self, SoupMessage *message)
 	}
 
 	g_mutex_unlock (&(priv->write_mutex));
-
-	soup_session_unpause_message (priv->session, priv->message);
 }
 
 static void
@@ -1100,8 +1102,6 @@ upload_thread (GDataUploadStream *self)
 {
 	GDataUploadStreamPrivate *priv = self->priv;
 
-	g_object_ref (self);
-
 	g_assert (priv->cancellable != NULL);
 
 	while (TRUE) {
@@ -1240,6 +1240,7 @@ finished_outer:
 	g_cond_signal (&(priv->finished_cond));
 	g_mutex_unlock (&(priv->response_mutex));
 
+	/* Referenced in create_network_thread(). */
 	g_object_unref (self);
 
 	return NULL;
@@ -1251,6 +1252,7 @@ create_network_thread (GDataUploadStream *self, GError **error)
 	GDataUploadStreamPrivate *priv = self->priv;
 
 	g_assert (priv->network_thread == NULL);
+	g_object_ref (self); /* ownership transferred to thread */
 	priv->network_thread = g_thread_try_new ("upload-thread", (GThreadFunc) upload_thread, self, error);
 }
 
